#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Source code for the used DNN model.

Submitted 6.11.2020 to Machine Learning and Knowledge Extraction as
supplementary material for manuscript:
    Do Randomized Algorithms Improve the Efficiency of Minimal Learning Machine?

@author: Joakim Linja

Package versions:

Keras==2.3.1
Keras-Applications==1.0.8
Keras-Preprocessing==1.1.2
numpy==1.16.4
tensorboard==2.1.1
tensorboard-plugin-wit==1.7.0
tensorflow==2.1.0
tensorflow-cpu==2.1.0
tensorflow-estimator==2.1.0
"""
import tensorflow as tf
import numpy as np
from typing import List
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense
from time import time, process_time


def times():
    return process_time(), time()


def compileFNN(inputShape: int, layers: List[int]):
    """
    """
    model = Sequential()
    model.add(tf.keras.Input(shape=inputShape))
    for layer in layers:
        model.add(Dense(layer, activation='sigmoid'))
    model.compile(optimizer='adam',
                  loss=tf.keras.metrics.mean_squared_error,
                  metrics=[tf.keras.metrics.RootMeanSquaredError()])
    return model


def trainFNN(model, epochs, batchSize, X, Y):
    """
    """
    model.fit(X, Y, epochs=epochs, batch_size=batchSize,
              callbacks=[tf.keras.callbacks.EarlyStopping(
                  monitor='root_mean_squared_error',
                  patience=10,
                  mode='auto')],
              use_multiprocessing=True,
              )
    return model


def testFNN(trX, trY, teX, teY, epochs=150, batchSize=10, layerSW=4):
    """
    """
    p0, w0 = times()

    (n,m) = np.shape(trX)
    inputShape = (n,)
    if layerSW == 4:
        layers = [int(m/2), int(m/4), int(m/8), 1]
    else:
        layers = [int(m/2), 1]
    model = compileFNN(inputShape, layers)

    p1, w1 = times()

    model = trainFNN(model, epochs, batchSize, trX, trY)

    p2, w2 = times()

    _, trainErr = model.evaluate(trX, trY)
    _, testErr = model.evaluate(teX, teY)

    p3, w3 = times()

    pts = [p1-p0, p2-p1, p3-p2, p3-p0]
    wts = [w1-w0, w2-w1, w3-w2, w3-w0]

    return model, trainErr, testErr, pts, wts
